// ---------------------------------------------------------------------------------------------------------
#include "main.h"

char c_err;				// Error Handler
unsigned int i1;				// Laufzeiger
unsigned char ca[9];


int main (void)
// ---------------------------------------------------------------------------
//					Demo-Programm zum Test des DS1820 Temperatursensors
// ---------------------------------------------------------------------------
{	
	// Input-Output der Ports initialisieren

	PORTA=0x00;
	DDRA=0x00;				// Port A einschalten. Hier sind die 5 Tasten angeschlossen.

	// Port B
	PORTB=0x00;				// Hier sind die LEDs angeschlossen
	DDRB=0xFF; 				// Alle Ports als Ausgnge setzen
	
	//Port C
	PORTC=0x00;				// Hier ist das LCD Display angeschlossen
	DDRC=0x00;				// Alle Ports als Ausgnge setzen

	// Port D
	PORTD=0x00;             // 
	DDRD |= (1<<DDD4);		// Tempsensor auf PD4

	// ----------------------------------------------------------------------------------------------------------------------

	lcd_init(12);			// erst mal das LCD Display einschalten - display on, cursor off
	lcd_command(0x01);		// display clear

	// ----------------------------------------------------------------------------------------------------------------------



	// -------------------------------------------- Reset ---------------------------------------------

	c_err = w1_reset();		
	if( c_err )	
	{
		return 0xff;	// Initialisierung des DS1820
		charaufLCD(0xff,14,1);		
	}

	//----------------------------------------------------------------------------------------------------

	w1_byte_wr( 0x33 );		// read ROM - jetzt kommt die 64 Bit Adresse zurck = 480008016590ba10
	
	ca[0] = w1_byte_rd();
	ca[1] = w1_byte_rd();
	ca[2] = w1_byte_rd();
	ca[3] = w1_byte_rd();
	ca[4] = w1_byte_rd();
	ca[5] = w1_byte_rd();
	ca[6] = w1_byte_rd();
	ca[7] = w1_byte_rd();

	//
	charaufLCD(ca[7],0,0);	
	charaufLCD(ca[6],2,0);	
	charaufLCD(ca[5],4,0);	
	charaufLCD(ca[4],6,0);	
	charaufLCD(ca[3],8,0);	
	charaufLCD(ca[2],10,0);	
	charaufLCD(ca[1],12,0);	
	charaufLCD(ca[0],14,0);	

	_delay_ms(1000);


	//----------------------------------------------------------------------------------------------------

	while(1)
	{
		c_err = w1_reset();		
		if( c_err )	
		{
			return 0xff;	// Initialisierung des DS1820
			charaufLCD(0xff,14,1);
			break;
		}
	//
		w1_byte_wr( 0xcc );		// skip ROM
		w1_byte_wr( 0x44 );		// convert
	//
	//
		//----------------------------------------------------------------------------------------------------
		c_err = w1_reset();		
		if( c_err )	
		{
			return 0xff;	// Initialisierung des DS1820
			charaufLCD(0xff,14,1);		
			break;
		}
		//
		w1_byte_wr( 0xcc );		// skip ROM
		w1_byte_wr( 0xbe );		// read Scatchpad
	//
		for ( i1=0 ; i1<9 ; i1++) ca[i1] = w1_byte_rd();
	//
		charaufLCD(ca[7],0,1);	
		charaufLCD(ca[6],2,1);	
		charaufLCD(ca[5],4,1);	
		charaufLCD(ca[4],6,1);	
		charaufLCD(ca[3],8,1);	
		charaufLCD(ca[2],10,1);	
		charaufLCD(ca[1],12,1);	
		charaufLCD(ca[0],14,1);	
	//
	}
	
	//----------------------------------------------------------------------------------------------------

	
	//------------------------ am Ende alle LEDs aufleuchten lassen ----------------------------
	PORTB = 0xFF;
	
	return 1;
}
//-------------------------------------------- end of main.c ------------------------------------------------------









//-------------------------------- Reset -----------------------------------------------------------------
char w1_reset(void)
{
	char err=0;

	DDRD |= 1<<DDD4;					// Port D Pin 4 auf Ausgang stellen - Tempsensor auf PD4 empfngt
	PORTD &= ~(1<<DDD4);				// PortD Pin 4 auf Null stellen
	_delay_us(480);						// 480 us Delay
	SREG &= ~(1<<7);					// oberstes Bit lschen = I-Bit Global Interrupt disable damit kein Interrupt in dieser Zeit stren kann
	DDRD &= ~(1<<DDD4);				// Port D Pin 4 auf Eingang = Tempsensor auf PD4 sendet nun zurck
	_delay_us(66);						// abwarten
	err = PIND & (1<<DDD4);			// (1<<DDD4) entspricht 0x10 - also 
	SREG |= (1<<7);					// oberstes Bit setzen = I-Bit Global Interrupt enable
	_delay_us(412);						// abwarten
	if( (PIND & (1<<DDD4)) == 0 )	err = 1;	// Kurzschlu, weil Datenleitung immer noch unten ist
	
	return err;
}
// ------------------------------- Ende Reset ------------------------------------------------------------







//------------------------------------------------------------------------------------------------------------
//Liest ein Byte
//-------------------------------- Byte read -----------------------------------------------------------------
unsigned char w1_byte_rd( void )
{
	//charaufLCD(0xcc,14,1);					// testweise - war hier drin
	return w1_byte_wr( 0xFF );					// bei Lesen muss Input b=1 auf jedem Bit 1 sein

}
//--------------------------------------------------------------------------------------------------------------






// ------------------------------------------------------------------------------------------------------------
// schreibt das Byte = b
//-------------------------------- Byte write -----------------------------------------------------------------
unsigned char w1_byte_wr( unsigned char b )
{
	unsigned char i=8 , j;
	//
	do
	{
		j = w1_bit_io( b & 0x01 );				// hier wird das niederwertigste Bit herausgezogen - nur das zhlt
		b >>= 1;								// ein Bit nach rechts schieben, um b abzuarbeiten
		if(j) b |= 0x80;						// hat ein Bit erkannt und schiebt oben in b ein Bit ein 10000000
	}
	while( --i );
	return b;
}
//--------------------------------------------------------------------------------------------------------------








//-------------------------------- Bit IO -----------------------------------------------------------------
// Hier wird ein Bit Schreiben oder Lesen ausgefhrt
//
// bei Lesen muss Input b=1 sein
//
// als Output kommt bei Lesen
// 1 oder 0 gelesen
//
// bei Schreiben wird  ber b eine 1 oder 0 bergeben
//---------------------------------------------------------------------------------------------------------
unsigned char w1_bit_io( unsigned char b )
{
	cli();										// Global Interrupt lschen
	DDRD |= (1<<DDD4);							// Pin PD4 auf Ausgang stellen
	PORTD &= ~(1<<DDD4);						// PortD Pin 4 auf Null stellen
	_delay_us(1);								// und 1 us Delay abwarten - das wre dann ein MASTER READ
	if(b) DDRD &= ~(1<<DDD4);					// ist b=1 , also Lesen gewnscht dann Pin PD4 als Eingang stellen 
	_delay_us(19);								// 14 us delay war etwas knapp, deshalb 20 us delay
	if( (PIND & (1<<DDD4)) == 0 )	b = 0;		// PIN PD4 ist unten also logische Null, dann wurde eine 0 gelesen
	_delay_us(40);								// 45 us Delay damit die 60 us erfllt sind.
	DDRD &= ~(1<<DDD4);						// Pin PD4 aus Eingang stellen
	sei();										// Global Interrupt setzen
	return b;									// und b zurckgeben
}
//------------------------------------------ end of w1_bit_io() -------------------------------------------








//--------------------------------------------------------------------------------------------------------------
//
// Sucht die vorhandenen Messsensoren
//
// 1. reset
// 2. ROM command = search ROM
//
//--------------------------------------------------------------------------------------------------------------
unsigned char w1_rom_search( unsigned char diff, int *id )
{
	unsigned char i , j , next_diff;	// j = Laufzeiger 
	char b;

	if( w1_reset() )	return 0xff;	// Reset und schauen // #define	PRESENCE_ERR	0xff
	w1_byte_wr( 0xf0 );					// ROM search command // #define	SEARCH_ROM	0xf0
	next_diff = 0x00;					// unchanged on last device // #define LAST_DEVICE	0x00
	i = 64;								// 8 bytes = 64 Bit - verteilen auf 8 Byte
	do
	{
		j = 8;							// 8 bits
		do
		{
			b = w1_bit_io( 1 );			// ein Bit lesen und auf b speichern
			if( w1_bit_io( 1 ) )
			{							// read complement bit
				if( b )					// hat eine 1 gelesen
				{
					return 0xfe;		// data error // #define	DATA_ERR	0xfe	
					charaufLCD(0xfe,14,1);	
				}
			}
			else
			{
				if( !b )
				{													// 00 = 2 devices
					if( diff > i || ((*id & 1) && diff != i) )
					{
						b = 1;										// now 1
						next_diff = i;								// next pass 0
					}
				}
			}
			w1_bit_io(b);											// write bit
			*id >>= 1;
			if( b )													// store bit
			*id |= 0x80;
			i--;
		}
		while( --j );
		id++;													// next byte
	}
	while(i);
	return next_diff;											// to continue search
}
//--------------------------------------------------------------------------------------------------------------




//----------------------------------------------------------------------------------------------------
//
// Hiermit wird veranlasst, da der Temparatursensor seinen Messwert in seinen Speicher bertrgt.
// w1_command is die Komplette Sequenz reset -> ROM command -> command
//
// vorher mssen aber die Messsonsoren gefunden werden
//
//----------------------------------------------------------------------------------------------------
void start_meas(void)
{
	if( PIND & (1<< DDD4) )			
	{									// DS1820 transmits conversion status to master
		w1_command(0x44,0);				// #define CONVERT_T = 0x44
		PORTD |= (1<< DDD4);			// PD4 hoch setzen
		DDRD |= (1<< DDD4);			// PD4 als Ausgang setzen
	}
	else
	{	
		charaufLCD(0xff,14,1);			// Fehlermeldung
	}
}
//------------------------------------------- end of start_meas() --------------------------------------------








// ------------------------------ Messwert lesen --------------------------------------------------------------
void read_meas(void)
{
	unsigned char id[8], diff;
	int *p_id;
	p_id = (int*)&id;

	unsigned char i;
	unsigned int temp;

	for( diff = 0xff; diff != 0x00; )					// #define LAST_DEVICE	0x00 
	//													// #define	SEARCH_FIRST	0xFF		// start new search
	{
		diff = w1_rom_search( diff, p_id );
		if( diff == 0xff )								// #define	PRESENCE_ERR	0xFF
		{
			charaufLCD(0xcc,14,1);						// Fehlermeldung:"No Sensor found"
			break;
		}
		if( diff == 0xfe )								// #define	DATA_ERR	0xFE
		{
			charaufLCD(0xdd,14,1);						// Fehlermeldung "Bus Error"
			break;
		}
		if( id[0] == 0x28 || id[0] == 0x10 )			// Typ: Temperatur Sensor DS18S20s 1-Wire family code: 0x10
		{	
			for( i = 0; i< 8 ; i++ )
			{
				PORTB=id[i];
				//charaufLCD(i,6,0);						// und auf LCD ansehen
				//charaufLCD(id[i],9,0);					// und auf LCD ansehen
				_delay_ms(200);
			}
			w1_byte_wr(0xbe);								// read command // #define READ		0xBE
			temp = w1_byte_rd();							// unteres Byte
			intaufLCD(temp,0,1);							// Ergebnis Temperatur auf LCD ansehen
			//temp |= (unsigned int)(w1_byte_rd() << 8);	// oberes Byte
			//if( id[0] == 0x10 )	temp <<= 3;				// 9 -> 12 bit
		}
	}
	//
}
//------------------------------------------- end of read_meas() --------------------------------------------















//------------------------------------------------------------------Command -------------------------------
//
// Erst die Initialisierung, dann ein ROM COMMAND und dann ein FUNCTION Command
// Fall die ID Null ist, wird das Command mit SKIP_ROM gleichzeitig an alle Sensoren geschickt.
//
//------------------------------------------------------------------Command -------------------------------
void w1_command( unsigned char command, int *id )
{
	unsigned char i;
	w1_reset();						// Erst die Intitialisierung

	if(id)
	{
		w1_byte_wr(0x55);			// MATCH ROM COMMAND 0x55 an einen einzelnen Sensor
		i = 8;						// The match ROM command followed by a 64-bit ROM code sequence allows the bus master 
		do							// to address a specific slave device on a multidrop or single-drop bus
		{
			w1_byte_wr(*id);		// Hier wird mitgeteilt welcher Sensor angesprochen werden soll
			id++;					 
		}							
		while( --i );				
	}
	else
	{
		w1_byte_wr(0xCC);			// SKIP_ROM	0xCC - The master can use this command to address all devices on the bus
	}								// simultaneously without sending out any ROM code information.
	//								// For example, the master can make all DS18S20s on the bus perform simultaneous 
	w1_byte_wr(command);			// temperature conversions.
}
//----------------------------------------------------------------------------------------------------------




